% SECTION 4.3 Intrinsic images
% to avoid MatLab convex optimization code we propose this faster
% implementation
%-----------------------------------------------------------------------
% solving Ax=b using x=inv(A)*b where x is [Mb Ms]' and A is [cb 0;0 cs]
% and b is Fi+D*P
% rgbSize is the n*m size of the image
%-----------------------------------------------------------------------
function [Mb Ms]=solveMbMsPositive(f,cb,cs)
    
    % Here we want to only fix those pixels whos mb or ms (or both)
    % are negative (so we build a mask)
    A=[cb',cs'];
    A=real(A);
    ainv=pinv(A);
    mbms=ainv * f';
    
    % First those whose ms is negative (by putting them to zero and figuring
    % out the corresponding mb)
    sMask=sign(mbms(2,:))<0;
    indx=find(sMask);
    b=f(indx,:)';
    mx=pinv(cb')*b;

    Mb=mbms(1,:);
    Ms=mbms(2,:);
    Mb(indx)=mx;
    Ms(indx)=zeros(size(indx,1));

    % Second those whose mb is negative (by putting them to zero and figuring
    % out the corresponding ms)
    bMask=sign(mbms(1,:)) <0;
    indx=find(bMask); b=f(indx,:)';
    mx=pinv(cs')*b;

    Ms(indx)=mx;
    Mb(indx)=zeros(size(indx,1));

end
